--
-- ORTHOINSPECTOR PostgreSQL database schema
--


--SET statement_timeout = 0;
--SET client_encoding = 'UTF8';
--SET standard_conforming_strings = off;
--SET check_function_bodies = false;
--SET client_min_messages = warning;
--SET escape_string_warning = off;
--SET search_path = public, pg_catalog;
--SET default_tablespace = '';
--SET default_with_oids = false;


--
-- Table structures
--


CREATE TABLE bank (
    pk_bank integer NOT NULL,
    identifier character varying(255),
    description character varying(255)
);
CREATE TABLE best_hit (
    pk_bh bigint NOT NULL,
    pk_sequencea integer,
    pk_sequenceb integer,
    pk_organisma integer,
    pk_organismb integer,
    pk_version integer
);
CREATE TABLE blast (
    pk_blast integer NOT NULL,
    pk_version integer,
    pk_sequence integer
);
CREATE TABLE hit (
    pk_blast integer,
    pk_sequence integer,
    pk_version integer,
    percent_id real,
    align_length integer,
    mismatches integer,
    gap_open integer,
    q_start integer,
    q_end integer,
    s_start integer,
    s_end integer,
    evalue double precision,
    bitscore real
);
CREATE TABLE inparalog (
    pk_inparalog bigint NOT NULL,
    pk_organism integer,
    pk_organism_rapporteur integer,
    pk_version integer
);
CREATE TABLE ln_inparalog_sequence (
    pk_inparalog bigint,
    pk_sequence integer
);
CREATE TABLE ln_organism_bank (
    pk_bank integer,
    pk_organism integer,
    taxid integer,
    date date
);
CREATE TABLE ln_organism_sequence (
    pk_organism integer,
    pk_sequence integer
);
CREATE TABLE ln_putative_inparalog_sequence (
    pk_putative_inparalog bigint,
    pk_sequence integer
);
CREATE TABLE manytomany (
    pk_manytomany bigint NOT NULL,
    pk_inparaloga bigint,
    pk_inparalogb bigint,
    pk_organisma integer,
    pk_organismb integer
);
CREATE TABLE no_hit (
    pk_sequence integer,
    pk_version integer
);
CREATE TABLE no_hit_to_orga (
    pk_organism integer,
    pk_sequence integer,
    pk_version integer
);
CREATE TABLE onetomany (
    pk_onetomany bigint NOT NULL,
    pk_sequencea integer,
    pk_inparalogb bigint,
    pk_organisma integer,
    pk_organismb integer
);
CREATE TABLE onetoone (
    pk_onetoone bigint NOT NULL,
    pk_sequencea integer,
    pk_sequenceb integer,
    pk_organisma integer,
    pk_organismb integer
);
CREATE TABLE organism (
    pk_organism integer NOT NULL,
    identifier character varying(255),
    genus character varying(255),
    species character varying(255),
    phylum character varying(2048)
);
CREATE TABLE putative_inparalog (
    pk_putative_inparalog bigint NOT NULL,
    pk_sequence integer,
    pk_organism integer,
    pk_organism_rapporteur integer,
    pk_version integer
);
CREATE TABLE sequences (
    pk_sequence integer NOT NULL,
    pk_version integer,
    checksum character varying(64),
    description text,
    sequence text,
    identifier character varying(255)
);
CREATE TABLE version (
    pk_version integer NOT NULL,
    date date,
    type integer,
    description text
);


--
-- Sequences
--


CREATE SEQUENCE sequences_pk_sequence_seq
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;
CREATE SEQUENCE version_pk_version_seq
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;
CREATE SEQUENCE organism_pk_organism_seq
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;
CREATE SEQUENCE bank_pk_bank_seq
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;

--
-- Contraints generated at first launch, during database creation
--

ALTER SEQUENCE sequences_pk_sequence_seq OWNED BY sequences.pk_sequence;
ALTER SEQUENCE version_pk_version_seq OWNED BY version.pk_version;
ALTER SEQUENCE organism_pk_organism_seq OWNED BY organism.pk_organism;
ALTER SEQUENCE bank_pk_bank_seq OWNED BY bank.pk_bank;

ALTER TABLE bank ALTER COLUMN pk_bank SET DEFAULT nextval('bank_pk_bank_seq'::regclass);
ALTER TABLE organism ALTER COLUMN pk_organism SET DEFAULT nextval('organism_pk_organism_seq'::regclass);
ALTER TABLE sequences ALTER COLUMN pk_sequence SET DEFAULT nextval('sequences_pk_sequence_seq'::regclass);
ALTER TABLE version ALTER COLUMN pk_version SET DEFAULT nextval('version_pk_version_seq'::regclass);

ALTER TABLE ONLY bank
    ADD CONSTRAINT pk_bank PRIMARY KEY (pk_bank);
ALTER TABLE ONLY organism
    ADD CONSTRAINT pk_organism PRIMARY KEY (pk_organism);
ALTER TABLE ONLY sequences
    ADD CONSTRAINT pk_sequences PRIMARY KEY (pk_sequence);
ALTER TABLE ONLY version
    ADD CONSTRAINT pk_version PRIMARY KEY (pk_version);


CREATE INDEX index_sequences_checksum ON sequences USING btree (checksum);
CREATE INDEX index_sequences_identifier ON sequences USING btree (identifier);
CREATE INDEX index_sequences_pk_sequence ON sequences USING btree (pk_sequence);

--
-- Contraints for "organism update" step
--


ALTER TABLE ONLY ln_organism_bank
    ADD CONSTRAINT fk_ln_organ_reference_bank FOREIGN KEY (pk_bank) REFERENCES bank(pk_bank) ON UPDATE RESTRICT ON DELETE RESTRICT;
ALTER TABLE ONLY ln_organism_bank
    ADD CONSTRAINT fk_ln_organ_reference_organism3 FOREIGN KEY (pk_organism) REFERENCES organism(pk_organism) ON UPDATE RESTRICT ON DELETE RESTRICT;

ALTER TABLE ONLY ln_organism_sequence
    ADD CONSTRAINT fk_ln_organ_reference_organism2 FOREIGN KEY (pk_organism) REFERENCES organism(pk_organism) ON UPDATE RESTRICT ON DELETE RESTRICT;
ALTER TABLE ONLY ln_organism_sequence
    ADD CONSTRAINT fk_ln_organ_reference_sequence FOREIGN KEY (pk_sequence) REFERENCES sequences(pk_sequence) ON UPDATE RESTRICT ON DELETE RESTRICT;

ALTER TABLE ONLY sequences
    ADD CONSTRAINT fk_sequence_reference_version FOREIGN KEY (pk_version) REFERENCES version(pk_version) ON UPDATE RESTRICT ON DELETE RESTRICT;

--CREATE INDEX index_sequences_checksum ON sequences USING btree (checksum);
--CREATE INDEX index_sequences_identifier ON sequences USING btree (identifier);
--CREATE INDEX index_sequences_pk_sequence ON sequences USING btree (pk_sequence);

CREATE INDEX ln_organism_sequence_pk_organism ON ln_organism_sequence USING btree (pk_organism);
CREATE INDEX ln_organism_sequence_pk_sequence ON ln_organism_sequence USING btree (pk_sequence);


--
-- Contraints for "blast parsing" step
--

ALTER TABLE ONLY best_hit
    ADD CONSTRAINT pk_best_hit PRIMARY KEY (pk_bh);
ALTER TABLE ONLY putative_inparalog
    ADD CONSTRAINT pk_putative_inparalog PRIMARY KEY (pk_putative_inparalog);


ALTER TABLE ONLY best_hit
    ADD CONSTRAINT fk_best_hit_ref48 FOREIGN KEY (pk_sequencea) REFERENCES sequences(pk_sequence) ON UPDATE RESTRICT ON DELETE RESTRICT;
ALTER TABLE ONLY best_hit
    ADD CONSTRAINT fk_best_hit_ref49 FOREIGN KEY (pk_sequenceb) REFERENCES sequences(pk_sequence) ON UPDATE RESTRICT ON DELETE RESTRICT;
ALTER TABLE ONLY best_hit
    ADD CONSTRAINT fk_best_hit_reference_organism5 FOREIGN KEY (pk_organisma) REFERENCES organism(pk_organism) ON UPDATE RESTRICT ON DELETE RESTRICT;
ALTER TABLE ONLY best_hit
    ADD CONSTRAINT fk_best_hit_reference_organism6 FOREIGN KEY (pk_organismb) REFERENCES organism(pk_organism) ON UPDATE RESTRICT ON DELETE RESTRICT;
ALTER TABLE ONLY best_hit
    ADD CONSTRAINT fk_best_hit_reference_version FOREIGN KEY (pk_version) REFERENCES version(pk_version) ON UPDATE RESTRICT ON DELETE RESTRICT;

ALTER TABLE ONLY putative_inparalog
    ADD CONSTRAINT fk_putative_ref10 FOREIGN KEY (pk_organism_rapporteur) REFERENCES organism(pk_organism) ON UPDATE RESTRICT ON DELETE RESTRICT;
ALTER TABLE ONLY putative_inparalog
    ADD CONSTRAINT fk_putative_reference_organism9 FOREIGN KEY (pk_organism) REFERENCES organism(pk_organism) ON UPDATE RESTRICT ON DELETE RESTRICT;
ALTER TABLE ONLY putative_inparalog
    ADD CONSTRAINT fk_putative_reference_sequence7 FOREIGN KEY (pk_sequence) REFERENCES sequences(pk_sequence) ON UPDATE RESTRICT ON DELETE RESTRICT;
ALTER TABLE ONLY putative_inparalog
    ADD CONSTRAINT fk_putative_reference_version FOREIGN KEY (pk_version) REFERENCES version(pk_version) ON UPDATE RESTRICT ON DELETE RESTRICT;


ALTER TABLE ONLY ln_putative_inparalog_sequence
    ADD CONSTRAINT fk_ln_putat_reference_putative FOREIGN KEY (pk_putative_inparalog) REFERENCES putative_inparalog(pk_putative_inparalog) ON UPDATE RESTRICT ON DELETE RESTRICT;
-- the following index is finally useless, as during putative_inparalog analysis, only only column is used for join
-- after the installation the table is not used anymore
--ALTER TABLE ONLY ln_putative_inparalog_sequence
--    ADD CONSTRAINT fk_ln_putat_reference_sequence FOREIGN KEY (pk_sequence) REFERENCES sequences(pk_sequence) ON UPDATE RESTRICT ON DELETE RESTRICT;

CREATE INDEX best_hit_organisma ON best_hit USING btree (pk_organisma);
CREATE INDEX best_hit_organismb ON best_hit USING btree (pk_organismb);
CREATE INDEX best_hit_pk_sequencea ON best_hit USING btree (pk_sequencea);
CREATE INDEX best_hit_pk_sequenceb ON best_hit USING btree (pk_sequenceb);


CREATE INDEX index_putative_inparalog_pk_organism ON putative_inparalog USING btree (pk_organism);
CREATE INDEX index_putative_inparalog_pk_organism_rapporteur ON putative_inparalog USING btree (pk_organism_rapporteur);
CREATE INDEX index_putative_inparalog_pk_sequence ON putative_inparalog USING btree (pk_sequence);

-- the following index is finally useless, as during putative_inparalog analysis, only only column is used for join
-- after the installation the table is not used anymore
--CREATE INDEX index_putative_inparalog_sequence_pk_sequence ON ln_putative_inparalog_sequence USING btree (pk_sequence);
CREATE INDEX index_putative_inparalog_sequence_pk_putative_inparalog ON ln_putative_inparalog_sequence USING btree (pk_putative_inparalog);

--
-- Contraints for "blast parsing" step, used if the option "create blast tables" option is selected
-- These tables are used only for the "best-hit density graph" tool of the graphical interface
--

ALTER TABLE ONLY blast
    ADD CONSTRAINT pk_blast PRIMARY KEY (pk_blast);

ALTER TABLE ONLY blast
    ADD CONSTRAINT fk_blast_reference_sequence FOREIGN KEY (pk_sequence) REFERENCES sequences(pk_sequence) ON UPDATE RESTRICT ON DELETE RESTRICT;
ALTER TABLE ONLY blast
    ADD CONSTRAINT fk_blast_reference_version FOREIGN KEY (pk_version) REFERENCES version(pk_version) ON UPDATE RESTRICT ON DELETE RESTRICT;

ALTER TABLE ONLY hit
    ADD CONSTRAINT fk_hit_reference_blast FOREIGN KEY (pk_blast) REFERENCES blast(pk_blast) ON UPDATE RESTRICT ON DELETE RESTRICT;
ALTER TABLE ONLY hit
    ADD CONSTRAINT fk_hit_reference_sequence FOREIGN KEY (pk_sequence) REFERENCES sequences(pk_sequence) ON UPDATE RESTRICT ON DELETE RESTRICT;
ALTER TABLE ONLY hit
    ADD CONSTRAINT fk_hit_reference_version FOREIGN KEY (pk_version) REFERENCES version(pk_version) ON UPDATE RESTRICT ON DELETE RESTRICT;

CREATE INDEX blast_pk_sequence ON blast USING btree (pk_sequence);

CREATE INDEX hit_bitscore ON hit USING btree (bitscore);
CREATE INDEX hit_evalue ON hit USING btree (evalue);
CREATE INDEX hit_pk_blast ON hit USING btree (pk_blast);
CREATE INDEX hit_pk_sequence ON hit USING btree (pk_sequence);

--
-- Contraints for "blast parsing" step, used if the option "create no_hit/no_hit_to_orga tables" option is selected
-- These tables are used only for the "extended statistics" tool of the graphical interface
--

ALTER TABLE ONLY no_hit
    ADD CONSTRAINT fk_no_hit_reference_sequence FOREIGN KEY (pk_sequence) REFERENCES sequences(pk_sequence) ON UPDATE RESTRICT ON DELETE RESTRICT;
ALTER TABLE ONLY no_hit
    ADD CONSTRAINT fk_no_hit_reference_version FOREIGN KEY (pk_version) REFERENCES version(pk_version) ON UPDATE RESTRICT ON DELETE RESTRICT;

ALTER TABLE ONLY no_hit_to_orga
    ADD CONSTRAINT fk_no_hit_t_reference_organism FOREIGN KEY (pk_organism) REFERENCES organism(pk_organism) ON UPDATE RESTRICT ON DELETE RESTRICT;
ALTER TABLE ONLY no_hit_to_orga
    ADD CONSTRAINT fk_no_hit_t_reference_sequence FOREIGN KEY (pk_sequence) REFERENCES sequences(pk_sequence) ON UPDATE RESTRICT ON DELETE RESTRICT;
ALTER TABLE ONLY no_hit_to_orga
    ADD CONSTRAINT fk_no_hit_t_reference_version FOREIGN KEY (pk_version) REFERENCES version(pk_version) ON UPDATE RESTRICT ON DELETE RESTRICT;

CREATE INDEX no_hit_pk_sequence ON no_hit USING btree (pk_sequence);

CREATE INDEX no_hit_to_orga_pk_organism ON no_hit_to_orga USING btree (pk_organism);
CREATE INDEX no_hit_to_orga_pk_sequence ON no_hit_to_orga USING btree (pk_sequence);

--
-- Contraints for "inparalogy validation" step
--

ALTER TABLE ONLY inparalog
    ADD CONSTRAINT pk_inparalog PRIMARY KEY (pk_inparalog);

ALTER TABLE ONLY inparalog
    ADD CONSTRAINT fk_inparalo_ref11 FOREIGN KEY (pk_organism) REFERENCES organism(pk_organism) ON UPDATE RESTRICT ON DELETE RESTRICT;
ALTER TABLE ONLY inparalog
    ADD CONSTRAINT fk_inparalo_ref12 FOREIGN KEY (pk_organism_rapporteur) REFERENCES organism(pk_organism) ON UPDATE RESTRICT ON DELETE RESTRICT;
ALTER TABLE ONLY inparalog
    ADD CONSTRAINT fk_inparalo_reference_version FOREIGN KEY (pk_version) REFERENCES version(pk_version) ON UPDATE RESTRICT ON DELETE RESTRICT;

ALTER TABLE ONLY ln_inparalog_sequence
    ADD CONSTRAINT fk_ln_inpar_reference_inparalo FOREIGN KEY (pk_inparalog) REFERENCES inparalog(pk_inparalog) ON UPDATE RESTRICT ON DELETE RESTRICT;
ALTER TABLE ONLY ln_inparalog_sequence
    ADD CONSTRAINT fk_ln_inpar_reference_sequence FOREIGN KEY (pk_sequence) REFERENCES sequences(pk_sequence) ON UPDATE RESTRICT ON DELETE RESTRICT;


CREATE INDEX index_inparalog_pk_organism ON inparalog USING btree (pk_organism);
CREATE INDEX index_inparalog_pk_organism_rapporteur ON inparalog USING btree (pk_organism_rapporteur);

CREATE INDEX index_ln_inparalog_sequence_pk_inparalog ON ln_inparalog_sequence USING btree (pk_inparalog);
CREATE INDEX index_ln_inparalog_sequence_pk_sequence ON ln_inparalog_sequence USING btree (pk_sequence);


--
-- Contraints for "orthology calculation" step
--

ALTER TABLE ONLY manytomany
    ADD CONSTRAINT pk_manytomany PRIMARY KEY (pk_manytomany);
ALTER TABLE ONLY onetomany
    ADD CONSTRAINT pk_onetomany PRIMARY KEY (pk_onetomany);
ALTER TABLE ONLY onetoone
    ADD CONSTRAINT pk_onetoone PRIMARY KEY (pk_onetoone);

ALTER TABLE ONLY onetoone
    ADD CONSTRAINT fk_onetoone_ref120 FOREIGN KEY (pk_sequencea) REFERENCES sequences(pk_sequence) ON UPDATE RESTRICT ON DELETE RESTRICT;
ALTER TABLE ONLY onetoone
    ADD CONSTRAINT fk_onetoone_ref18 FOREIGN KEY (pk_organisma) REFERENCES organism(pk_organism) ON UPDATE RESTRICT ON DELETE RESTRICT;
ALTER TABLE ONLY onetoone
    ADD CONSTRAINT fk_onetoone_ref19 FOREIGN KEY (pk_organismb) REFERENCES organism(pk_organism) ON UPDATE RESTRICT ON DELETE RESTRICT;
ALTER TABLE ONLY onetoone
    ADD CONSTRAINT fk_onetoone_ref56 FOREIGN KEY (pk_sequenceb) REFERENCES sequences(pk_sequence) ON UPDATE RESTRICT ON DELETE RESTRICT;

ALTER TABLE ONLY manytomany
    ADD CONSTRAINT fk_manytoma_ref13 FOREIGN KEY (pk_inparaloga) REFERENCES inparalog(pk_inparalog) ON UPDATE RESTRICT ON DELETE RESTRICT;
ALTER TABLE ONLY manytomany
    ADD CONSTRAINT fk_manytoma_ref14 FOREIGN KEY (pk_inparalogb) REFERENCES inparalog(pk_inparalog) ON UPDATE RESTRICT ON DELETE RESTRICT;
ALTER TABLE ONLY manytomany
    ADD CONSTRAINT fk_manytoma_ref15 FOREIGN KEY (pk_organisma) REFERENCES organism(pk_organism) ON UPDATE RESTRICT ON DELETE RESTRICT;
ALTER TABLE ONLY manytomany
    ADD CONSTRAINT fk_manytoma_ref16 FOREIGN KEY (pk_organismb) REFERENCES organism(pk_organism) ON UPDATE RESTRICT ON DELETE RESTRICT;

ALTER TABLE ONLY onetomany
    ADD CONSTRAINT fk_onetoman_ref20 FOREIGN KEY (pk_organisma) REFERENCES organism(pk_organism) ON UPDATE RESTRICT ON DELETE RESTRICT;
ALTER TABLE ONLY onetomany
    ADD CONSTRAINT fk_onetoman_ref21 FOREIGN KEY (pk_organismb) REFERENCES organism(pk_organism) ON UPDATE RESTRICT ON DELETE RESTRICT;
ALTER TABLE ONLY onetomany
    ADD CONSTRAINT fk_onetoman_reference_inparalo FOREIGN KEY (pk_inparalogb) REFERENCES inparalog(pk_inparalog) ON UPDATE RESTRICT ON DELETE RESTRICT;
ALTER TABLE ONLY onetomany
    ADD CONSTRAINT fk_onetoman_reference_sequence FOREIGN KEY (pk_sequencea) REFERENCES sequences(pk_sequence) ON UPDATE RESTRICT ON DELETE RESTRICT;


CREATE INDEX onetomany_pk_inparalogb ON onetomany USING btree (pk_inparalogb);
CREATE INDEX onetomany_pk_organisma ON onetomany USING btree (pk_organisma);
CREATE INDEX onetomany_pk_organismb ON onetomany USING btree (pk_organismb);
CREATE INDEX onetomany_pk_sequencea ON onetomany USING btree (pk_sequencea);

CREATE INDEX onetoone_pk_organisma ON onetoone USING btree (pk_organisma);
CREATE INDEX onetoone_pk_organismb ON onetoone USING btree (pk_organismb);
CREATE INDEX onetoone_pk_sequencea ON onetoone USING btree (pk_sequencea);
CREATE INDEX onetoone_pk_sequenceb ON onetoone USING btree (pk_sequenceb);

CREATE INDEX manytomany_inparaloga ON manytomany USING btree (pk_inparaloga);
CREATE INDEX manytomany_inparalogb ON manytomany USING btree (pk_inparalogb);
CREATE INDEX manytomany_organisma ON manytomany USING btree (pk_organisma);
CREATE INDEX manytomany_organismb ON manytomany USING btree (pk_organismb);



