-
-- Base de données: `orthoinspector`
--


--
-- Structure de la table `version`
--
PRAGMA foreign_keys = ON;

CREATE TABLE version(
    pk_version      INTEGER PRIMARY KEY NOT NULL,
    date            NUMERIC,
    type            INTEGER,
    description TEXT
);

CREATE TABLE bank (
    pk_bank         INTEGER PRIMARY KEY NOT NULL,
    identifier      VARCHAR(255),
    description     VARCHAR(255)
);

CREATE TABLE organism (
    pk_organism     INTEGER PRIMARY KEY NOT NULL,
    identifier      VARCHAR(255),
    genus           VARCHAR(255),
    species         VARCHAR(255),
    phylum          TEXT
);

CREATE TABLE sequences (
    pk_sequence     INTEGER PRIMARY KEY NOT NULL,
    pk_version      INTEGER,
    checksum        VARCHAR(64),
    description     TEXT,
    sequence        TEXT,
    identifier      VARCHAR(255),
    FOREIGN KEY (pk_version)      REFERENCES  version(pk_version)
);

CREATE TABLE best_hit (
    pk_bh           INTEGER PRIMARY KEY NOT NULL,
    pk_sequencea    INTEGER,
    pk_sequenceb    INTEGER,
    pk_organisma   INTEGER,
    pk_organismb   INTEGER,
    pk_version      INTEGER,
    FOREIGN KEY (pk_sequencea)    REFERENCES  sequences(pk_sequence),
    FOREIGN KEY (pk_sequenceb)    REFERENCES  sequences(pk_sequence),
    FOREIGN KEY (pk_organisma)    REFERENCES  organism(pk_organism),
    FOREIGN KEY (pk_organismb)    REFERENCES  organism(pk_organism),
    FOREIGN KEY (pk_version)      REFERENCES  version(pk_version)
 );
CREATE TABLE blast (
    pk_blast        INTEGER PRIMARY KEY NOT NULL,
    pk_version      INTEGER,
    pk_sequence     INTEGER,
    FOREIGN KEY (pk_version)      REFERENCES  version(pk_version),
    FOREIGN KEY (pk_sequence)     REFERENCES  sequences(pk_sequence)
);

CREATE  TABLE   putative_inparalog(
    pk_putative_inparalog   INTEGER PRIMARY KEY NOT NULL,
    pk_sequence             INTEGER,
    pk_organism             INTEGER,
    pk_organism_rapporteur  INTEGER,
    pk_version              INTEGER,
    FOREIGN KEY (pk_sequence)             REFERENCES  sequences(pk_sequence),
    FOREIGN KEY (pk_organism)             REFERENCES  organism(pk_organism),
    FOREIGN KEY (pk_organism_rapporteur)  REFERENCES  organism(pk_organism),
    FOREIGN KEY (pk_version)              REFERENCES  version(pk_version)
);

CREATE TABLE inparalog (
    pk_inparalog            INTEGER PRIMARY KEY NOT NULL,
    pk_organism             INTEGER,
    pk_organism_rapporteur  INTEGER,
    pk_version              INTEGER,
    FOREIGN KEY (pk_organism)             REFERENCES  organism(pk_organism),
    FOREIGN KEY (pk_organism_rapporteur)  REFERENCES  organism(pk_organism),
    FOREIGN KEY (pk_version)              REFERENCES  version(pk_version)
);

CREATE TABLE manytomany (
    pk_manytomany   INTEGER PRIMARY KEY NOT NULL,
    pk_inparaloga   INTEGER,
    pk_inparalogb   INTEGER,
    pk_organisma    INTEGER,
    pk_organismb    INTEGER,
    FOREIGN KEY (pk_inparaloga)   REFERENCES  inparalog(pk_inparalog),
    FOREIGN KEY (pk_inparalogb)   REFERENCES  inparalog(pk_inparalog),
    FOREIGN KEY (pk_organisma)    REFERENCES  organism(pk_organism),
    FOREIGN KEY (pk_organismb)    REFERENCES  organism(pk_organism)
);
CREATE TABLE onetomany (
    pk_onetomany    INTEGER PRIMARY KEY NOT NULL,
    pk_sequencea    INTEGER,
    pk_inparalogb   INTEGER,
    pk_organisma    INTEGER,
    pk_organismb    INTEGER,
    FOREIGN KEY (pk_sequencea)    REFERENCES  sequences(pk_sequence),
    FOREIGN KEY (pk_inparalogb)   REFERENCES  inparalog(pk_inparalog),
    FOREIGN KEY (pk_organisma)    REFERENCES  organism(pk_organism),
    FOREIGN KEY (pk_organismb)    REFERENCES  organism(pk_organism)
);
CREATE TABLE onetoone (
    pk_onetoone     INTEGER PRIMARY KEY NOT NULL,
    pk_sequencea    INTEGER,
    pk_sequenceb    INTEGER,
    pk_organisma    INTEGER,
    pk_organismb    INTEGER,
    FOREIGN KEY (pk_sequencea)    REFERENCES  sequences(pk_sequence),
    FOREIGN KEY (pk_sequenceb)    REFERENCES  sequences(pk_sequence),
    FOREIGN KEY (pk_organisma)    REFERENCES  organism(pk_organism),
    FOREIGN KEY (pk_organismb)    REFERENCES  organism(pk_organism)
);

CREATE TABLE hit (
    pk_blast        INTEGER,
    pk_sequence     INTEGER,
    pk_version      INTEGER,
    percent_id      REAL,
    align_length    INTEGER,
    mismatches      INTEGER,
    gap_open        INTEGER,
    q_start         INTEGER,
    q_end           INTEGER,
    s_start         INTEGER,
    s_end           INTEGER,
    evalue          REAL,
    bitscore        REAL,
    FOREIGN KEY (pk_blast)    REFERENCES  blast(pk_blast),
    FOREIGN KEY (pk_sequence) REFERENCES  sequences(pk_sequence),
    FOREIGN KEY (pk_version)  REFERENCES  version(pk_version)
);

CREATE TABLE ln_inparalog_sequence (
    pk_inparalog    INTEGER,
    pk_sequence     INTEGER,
    FOREIGN KEY (pk_inparalog)   REFERENCES  inparalog(pk_inparalog),
    FOREIGN KEY (pk_sequence)    REFERENCES  sequences(pk_sequence)
);

CREATE TABLE ln_putative_inparalog_sequence (
    pk_putative_inparalog    INTEGER,
    pk_sequence     INTEGER,
    FOREIGN KEY (pk_putative_inparalog)   REFERENCES  inparalog(pk_putative_inparalog),
    FOREIGN KEY (pk_sequence)    REFERENCES  sequences(pk_sequence)
);


CREATE TABLE ln_organism_sequence (
    pk_organism     INTEGER,
    pk_sequence     INTEGER,
    FOREIGN KEY (pk_organism) REFERENCES  organism(pk_organism),
    FOREIGN KEY (pk_sequence) REFERENCES  sequences(pk_sequence)
);

CREATE TABLE ln_organism_bank(
    pk_organism     INTEGER,
    pk_bank         INTEGER,
    taxid           INTEGER,
    date            NUMERIC,
    FOREIGN KEY (pk_organism) REFERENCES organism(pk_organism),
    FOREIGN KEY (pk_bank) REFERENCES bank(pk_bank)
);

CREATE TABLE no_hit (
    pk_sequence     INTEGER,
    pk_version      INTEGER,
    FOREIGN KEY (pk_sequence) REFERENCES  sequences(pk_sequence),
    FOREIGN KEY (pk_version)  REFERENCES  version(pk_version)
);
CREATE TABLE no_hit_to_orga (
    pk_organism     INTEGER,
    pk_sequence     INTEGER,
    pk_version      INTEGER,
    FOREIGN KEY (pk_organism) REFERENCES  organism(pk_organism),
    FOREIGN KEY (pk_sequence) REFERENCES  sequences(pk_sequence),
    FOREIGN KEY (pk_version)  REFERENCES  version(pk_version)
);

--
-- Contraints generated at first launch, during database creation
--

CREATE INDEX index_sequences_identifier ON sequences (identifier);

--
-- Contraints for "organism update" step
--


CREATE INDEX ln_organism_sequence_pk_organism ON ln_organism_sequence (pk_organism);
CREATE INDEX ln_organism_sequence_pk_sequence ON ln_organism_sequence (pk_sequence);

--
-- Contraints for "blast parsing" step
--

CREATE INDEX best_hit_organisma ON best_hit (pk_organisma);
CREATE INDEX best_hit_organismb ON best_hit (pk_organismb);
CREATE INDEX best_hit_pk_sequencea ON best_hit (pk_sequencea);
CREATE INDEX best_hit_pk_sequenceb ON best_hit (pk_sequenceb);


CREATE INDEX index_putative_inparalog_pk_organism ON putative_inparalog (pk_organism);
CREATE INDEX index_putative_inparalog_pk_organism_rapporteur ON putative_inparalog (pk_organism_rapporteur);
CREATE INDEX index_putative_inparalog_pk_sequence ON putative_inparalog (pk_sequence);
CREATE INDEX index_putative_inparalog_sequence_pk_putative_inparalog ON ln_putative_inparalog_sequence (pk_putative_inparalog);

--
-- Contraints for "blast parsing" step, used if the option "create blast tables" option is selected
-- These tables are used only for the "best-hit density graph" tool of the graphical interface

CREATE INDEX blast_pk_sequence ON blast (pk_sequence);

CREATE INDEX hit_bitscore ON hit (bitscore);
CREATE INDEX hit_evalue ON hit (evalue);
CREATE INDEX hit_pk_blast ON hit (pk_blast);
CREATE INDEX hit_pk_sequence ON hit (pk_sequence);


--
-- Contraints for "blast parsing" step, used if the option "create no_hit/no_hit_to_orga tables" option is selected
-- These tables are used only for the "extended statistics" tool of the graphical interface
--

CREATE INDEX no_hit_pk_sequence ON no_hit (pk_sequence);
CREATE INDEX no_hit_to_orga_pk_organism ON no_hit_to_orga (pk_organism);
CREATE INDEX no_hit_to_orga_pk_sequence ON no_hit_to_orga (pk_sequence);


--
-- Contraints for "inparalogy validation" step
--


CREATE INDEX index_inparalog_pk_organism ON inparalog (pk_organism);
CREATE INDEX index_inparalog_pk_organism_rapporteur ON inparalog  (pk_organism_rapporteur);

CREATE INDEX index_ln_inparalog_sequence_pk_inparalog ON ln_inparalog_sequence (pk_inparalog);
CREATE INDEX index_ln_inparalog_sequence_pk_sequence ON ln_inparalog_sequence (pk_sequence);


--
-- Contraints for "orthology calculation" step
--

CREATE INDEX onetomany_pk_inparalogb ON onetomany (pk_inparalogb);
CREATE INDEX onetomany_pk_organisma ON onetomany (pk_organisma);
CREATE INDEX onetomany_pk_organismb ON onetomany (pk_organismb);
CREATE INDEX onetomany_pk_sequencea ON onetomany (pk_sequencea);

CREATE INDEX onetoone_pk_organisma ON onetoone (pk_organisma);
CREATE INDEX onetoone_pk_organismb ON onetoone (pk_organismb);
CREATE INDEX onetoone_pk_sequencea ON onetoone (pk_sequencea);
CREATE INDEX onetoone_pk_sequenceb ON onetoone (pk_sequenceb);

CREATE INDEX manytomany_inparaloga ON manytomany (pk_inparaloga);
CREATE INDEX manytomany_inparalogb ON manytomany (pk_inparalogb);
CREATE INDEX manytomany_organisma ON manytomany (pk_organisma);
CREATE INDEX manytomany_organismb ON manytomany (pk_organismb);

